'use strict';

var convertNodesToTree = function convertNodesToTree(nodes, index) {
    if (nodes[index] && nodes[index].node.nodeName === '#text') {
        return { node: nodes[index].node, children: [] };
    }
    var obj = { node: nodes[index].node, children: [] };
    for (var i = index; i < nodes.length; i += 1) {
        if (nodes[i].parentNode === obj.node) obj.children.push(convertNodesToTree(nodes, i));
    }
    return obj;
};

var convertTreeToHTML = function convertTreeToHTML(nodeObj, continueOlNumbering, offset) {
    if (nodeObj.node.nodeName === '#text') return nodeObj.node.nodeValueCut;

    var numbering = continueOlNumbering && nodeObj.node.nodeName === 'OL' ? ' start=' + (parseInt(nodeObj.node.attributes.start.nodeValue, 10) + offset) : '';

    var baseString = '<' + nodeObj.node.nodeName.toLowerCase() + numbering + '>';

    for (var i = 0; i < nodeObj.children.length; i += 1) {
        baseString += convertTreeToHTML(nodeObj.children[i], continueOlNumbering, offset);
    }

    baseString += '</' + nodeObj.node.nodeName.toLowerCase() + '>';
    return baseString;
};

var getHighlightText = function getHighlightText(range) {
    var continueOlNumbering = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;

    var getNextNode = function getNextNode(node) {
        var temp = node;
        if (temp.firstChild) return temp.firstChild;
        while (temp) {
            if (temp.nextSibling) return temp.nextSibling;
            temp = temp.parentNode;
        }
        return null;
    };

    var startNode = range.startContainer,
        endNode = range.endContainer,
        commonAncestorNode = range.commonAncestorContainer,
        startOffset = range.startOffset,
        endOffset = range.endOffset;


    var nodes = [];

    var offset = 0;

    if (startNode === commonAncestorNode) {
        if (startNode.nodeValue) {
            startNode.nodeValueCut = startNode.nodeValue.substring(startOffset, endOffset);
        }
        if (startNode.parentNode.nodeName !== 'LI') {
            nodes.push({ node: startNode.parentNode, parentNode: startNode.parentNode.parentNode });
        }
        nodes.push({ node: startNode, parentNode: startNode.parentNode });
    } else {
        if (startNode.nodeValue) {
            startNode.nodeValueCut = startNode.nodeValue.substring(startOffset);
        }

        if (endNode.nodeValue) endNode.nodeValueCut = endNode.nodeValue.substring(0, endOffset);

        for (var tempNode = startNode.parentNode; tempNode; tempNode = tempNode.parentNode) {
            tempNode.nodeValueCut = tempNode.nodeValue;
            nodes.push({ node: tempNode, parentNode: tempNode.parentNode });
            if (tempNode === commonAncestorNode) break;
        }

        nodes.reverse();

        for (var _tempNode = startNode; _tempNode; _tempNode = getNextNode(_tempNode)) {
            if (_tempNode) {
                if (_tempNode !== startNode && _tempNode !== endNode) {
                    _tempNode.nodeValueCut = _tempNode.nodeValue;
                }
                nodes.push({ node: _tempNode, parentNode: _tempNode.parentNode });
                if (_tempNode === endNode) break;
            }
        }
    }
    if (commonAncestorNode.nodeName === 'OL' || nodes[1] && nodes[1].node.nodeName === 'OL') {
        var _tempNode2 = startNode;
        while (_tempNode2.nodeName !== 'LI') {
            _tempNode2 = _tempNode2.parentNode;
        }while (_tempNode2.previousSibling && _tempNode2.previousSibling.nodeName === 'LI') {
            offset += 1;
            _tempNode2 = _tempNode2.previousSibling;
        }
    }

    var nodeTree = convertNodesToTree(nodes, 0);
    var html = convertTreeToHTML(nodeTree, continueOlNumbering, offset);
    return { html: html, nodeTree: nodeTree };
};

module.exports = getHighlightText;